clear, clc, close all

%% Parameter
   omega = 0.9; % Relaxation factor
   
   x0 = [0; 0; 0]; % Inital vector
   
   tol = 1e-6; % Tolerance for stopping criterion
   
   maxit = 50; % Maximum number of iterations
   
%% Linear system
   A = [4 0  2;  % Matrix
        0 5  2;
        5 4 10];
   
   b = [4; -3; 2]; % Right hand side
  
%% SOR Gauss-Seidel method
   [xSOR,resSOR] = mysor(A,b,x0,tol,maxit,omega);
   
%% Output
 % plotres(res1,'name1',res2,'name2',...)
   plotres(resSOR,'SOR');
   
%% Spectral radius
  
 % SOR-matrix
   D = diag(diag(A));
   L = tril(A,-1);
   R = triu(A,+1);
   
   N = 1000;
   h = 2/(N-1);
  
   rho = zeros(N,1);
   omegavec = rho;
   for cnt = 0:N-1
     omega = cnt*h;
     invDL = inv(D+omega*L);  
     MSOR = invDL*((1-omega)*D-omega*R);
     
     rho(cnt+1) = max(abs(eig(MSOR)));
     omegavec(cnt+1) = omega;
   end
   
   figure
   plot(omegavec,rho,'LineWidth',2)
   xlabel('\omega','FontSize',12)
   ylabel('\rho','FontSize',12)