clear, clc, close all

%% Parameters
   
   tol = 1e-6; % Tolerance for stopping criterion
   
   maxit = 50; % Maximum number of iterations
   
   omega = 0.5; % Relaxation factor
   
   nu1 = 5; % Presmoothing steps
   nu2 = 5; % Postsmoothing steps   
   
 % Exact solution and right hand side  
   k = 3;
   uex = @(x) sin(k*pi*x);
   rhs = @(x) k^2*pi^2*sin(k*pi*x);
   
 % Output parameters  
   iclick = 1;
   
%% Linear system
   l = 5; % finest level l = 1,2,... 
   
   Nf = 2^(l+1)-1; % Number of inner points (fine)
   Nc = 2^l-1;     % Number of inner points (coarse)
   
   dxf = 1/(Nf+1); % Mesh width (fine)
   dxc = 1/(Nc+1); % Mesh width (coarse)
   
   e = ones(Nf,1);
   Dxx = spdiags([e -2*e e],[-1 0 1],Nf,Nf); % 1D central difference
   Af = -Dxx/(dxf^2); % Matrix (fine)
   
   e = ones(Nc,1);
   Dxx = spdiags([e -2*e e],[-1 0 1],Nc,Nc); % 1D central difference
   Ac = -Dxx/(dxc^2); % Matrix (coarse)
   
 
   x = dxf*(1:Nf)'; % Mesh (fine)
   f = rhs(x); % Right hand side (fine)
  
   u0 = rand(Nf,1); % Initial guess (fine)
   
%% Two-grid scheme

 % Restriction
   R = [kron(eye(Nc),[0,1]), zeros(Nc,1)];
   
 % Prolongation
   P = [kron(speye(Nc),[1 0]) + kron(speye(Nc),[0 2]),zeros(Nc,1)];
   P = P + [zeros(Nc,1) kron(speye(Nc),[0 1])];
   P = 0.5*P';
   
   mgplot(x,u0,u0-uex(x),[],[],[],iclick);
   
   u = u0;
   normres = norm(f-Af*u0,'inf');
   
   cnt = 1;
   resvec = zeros(maxit+1,1);
   resvec(1) = normres;
   while normres > tol && cnt <= maxit
     u = myjor(Af,f,u,nu1,omega); % Presmoothing steps
   
     mgplot(x,u,u-uex(x),[],[],[],iclick);
   
     deff = f - Af*u; % Defect (fine)
   
     mgplot(x,u,u-uex(x),deff,[],[],iclick);
   
     defc = R*deff; % Restriction of the defect
   
     mgplot(x,u,u-uex(x),deff,[],defc,iclick);
   
     errc = Ac\defc; % Solving on coarse mesh
   
     mgplot(x,u,u-uex(x),deff,errc,defc,iclick);
   
     errf = P*errc; % Prolongation of the error
   
     mgplot(x,u,errf,deff,errc,defc,iclick);
   
     u = u + errf; % Update solution
   
     mgplot(x,u,u-uex(x),deff,errc,defc,iclick);
   
     u = myjor(Af,f,u,nu2,omega); % Postsmoothing steps
   
     mgplot(x,u,u-uex(x),deff,errc,defc,iclick);  
     
     normres = norm(f-Af*u,'inf');
     cnt = cnt + 1; % increase counter
     resvec(cnt) = normres; % save residual     
   end
   resvec = resvec(1:cnt);
%% Output
 % plotres(res1,'name1',res2,'name2',...)
   plotres(resvec,'Twogrid')
   
 % plotsol(x,numsol,exsol)
   plotsol(x,u,uex);
   