clear, clc, close all

%% Parameters
   a = 0.5; % Advection parameter
   b = 2; % Reaction parameter
   
   tol = 1e-6; % Tolerance for stopping criterion
   
   maxit = 10000; % Maxium number of iterations
   
%% Linear system
   N = 99;
   dx = 1/(N+1);
   
   e = ones(N,1);
   Dxx = spdiags([e -2*e e],[-1 0 1],N,N); % 1D central difference
   Axx = kron(speye(N,N),Dxx)/(dx^2); % 2nd derivative with respect to x
   Ayy = kron(Dxx,speye(N,N))/(dx^2); % 2nd derivative with respect to y
   
   Dx = spdiags([-e e],[-1 1],N,N); % 1D central difference
   Ax= kron(speye(N,N),Dx)/(2*dx);
   
   I = speye(N^2);
   
   A = -Axx - Ayy -a*Ax - b*I; % Matrix   
   
   b = rand(N^2,1); % Right hand side
  
   u0 = ones(N^2,1); % Initial guess
   
%% GMRES method
   tic;
   [x,~,~,~,res1] = bicgstab(A,b,tol,maxit,[],[],u0);
   t1 = toc;
   
   tic;
   [~,~,~,~,res2] = bicgstab(A,b,tol,maxit,[],[],u0);
   t2 = toc;
   
   tic;
   [~,~,~,~,res3] = bicgstab(A,b,tol,maxit,[],[],u0);
   t3 = toc;
   
   tic;
   [~,~,~,~,res4] = bicgstab(A,b,tol,maxit,[],[],u0); 
   t4 = toc;   
   
   tic;
   [~,~,~,~,res5] = bicgstab(A,b,tol,maxit,[],[],u0);
   t5 = toc;
   
   tic;
   [~,~,~,~,res6] = bicgstab(A,b,tol,maxit,[],[],u0);
   t6 = toc;
   
   tic;
   [~,~,~,~,res7] = bicgstab(A,b,tol,maxit,[],[],u0);
   t7 = toc;
   
   tic;
   [~,~,~,~,res8] = bicgstab(A,b,tol,maxit,[],[],u0);
   t8 = toc;
   
%% Output
 % plotrestime(res1,time1,'name1,res2,time2,'name2',...)
   plotrestime(res1,t1,'method 1', ...
               res2,t2,'method 2', ...
               res3,t3,'method 3', ...
               res4,t4,'method 4', ...
               res5,t5,'method 5', ...
               res6,t6,'method 6', ...
               res7,t7,'method 7', ...
               res8,t8,'method 8');      