  function [x,resvec] = mybicg(A,b,x0,tol,maxit)
% BiCG method for the solution of a linear system
% Inputs:    A - system matrix
%            b - right hand side
%           x0 - initial guess
%          tol - method stops if the maximum norm of the residual
%                b-A*x is less than tol
%        maxit - maximum number of iterations
%
% Outputs:   x - solution vector
%       resvec - residual vector

  
% Init BiCG
  r = b - A*x0;
  rstar = r;
  p = r;
  pstar = r;
  normres = norm(r);
  
  x = x0;
  w = rstar'*r;
  
  resvec = zeros(maxit+1,1);
  resvec(1) = normres;
  
  cnt = 1;
  while normres > tol && cnt <= maxit
    v = A*p;
    vpstar = pstar'*v;       
    alpha = w/vpstar;
    x = x + alpha*p;
    r = r - alpha*v;
    vstar = A'*pstar;
    rstar = rstar - alpha*vstar;
    w1 = rstar'*r;
    beta = w1/w;
    w = w1;
    p = r + beta*p;
    pstar = rstar + beta*pstar;
    
    normres = norm(r);
    cnt = cnt + 1;
    resvec(cnt) = normres;
  end
  
  % shorten resvec
  resvec = resvec(1:cnt);
