  function [x,resvec] = mycgs(A,b,x0,tol,maxit)
% CGS method for the solution of a linear system
% Inputs:    A - system matrix
%            b - right hand side
%           x0 - initial guess
%          tol - method stops if the maximum norm of the residual
%                b-A*x is less than tol
%        maxit - maximum number of iterations
%
% Outputs:   x - solution vector
%       resvec - residual vector  
 
% Init CGS
  r0 = b - A*x0;
  r = r0;
  p = r;
  u = r;
  
  x = x0;
  w = r'*r0;

  normres = norm(r0);
  resvec = zeros(maxit+1,1);
  resvec(1) = normres;
  
  cnt = 1;
  while normres > tol && cnt <= maxit

%%% Algorithm

%%%
        
    normres = norm(r);
    cnt = cnt + 1;
    resvec(cnt) = normres;
  end
  
  % shorten resvec
  resvec = resvec(1:cnt);
