function [x,resvec] = mygraddesc(A,b,x0,tol,maxit)
% Gradient descent method for the solution of a linear system
% Inputs:    A - system matrix
%            b - right hand side
%           x0 - initial guess
%          tol - method stops if the maximum norm of the residual
%                b-A*x is less than tol         
%        maxit - maximum number of iterations
% 
% Outputs:   x - solution vector
%       resvec - residual vector

% initial residual  
  res = b-A*x0;
  normres = norm(b-A*x0); %2-norm
  
% save residual  
  resvec = zeros(maxit+1,1);
  resvec(1) = normres;

  x = x0; 
  cnt = 1;
  while normres > tol && cnt <= maxit
%##########################################################################      
% Algorithm



%##########################################################################      
    
    normres = norm(b-A*x); % residual
    cnt = cnt + 1; % increase counter
    resvec(cnt) = normres; % save residual
  end
  
% shorten resvec
  resvec = resvec(1:cnt);