  function plotrestime(res1,time1,name1,varargin)
% Call plotres(res1,time1,'name1',res2,time2,'name2',...)
% Inputs:
%   res1, res2, ... residual vectors
%   time1, time2 ... wall clock times
%   name1, name2, ... method names
  
  color = {'g','r','c','m','k', ...
           [255/255 140/255 0/255],[139/255 39/255 82/255],...
           [205/255 205/255 0/255]};
  
  if mod(nargin,3)~=0
    error ('Number of inputs must be a multiple of three');
  end
  if (nargin > 3*(length(color)+1))
    error('Too many input arguments. At most 7 methods allowed');  
  end

  n = nargin/3; % number of methods
  
  fprintf([' ',name1,' method:\n'])
  fprintf('  Iterations: %i\n',length(res1)-1);
  fprintf('  Residual:   %e\n',res1(end));
  fprintf('  Time:       %4.2fs\n\n',time1);
  
  figure
  subplot(1,2,1)
  semilogy(0:length(res1)-1,res1,'b','LineWidth',2)
  
  for cnt = 1:3:(nargin-3)-1    
    fprintf([' ',varargin{cnt+2},' method:\n'])
    fprintf('  Iterations: %i\n',length(varargin{cnt})-1);
    fprintf('  Residual:   %e\n',varargin{cnt}(end));
    fprintf('  Time:       %4.2fs\n\n',varargin{cnt+1});
    
    hold on
    semilogy(0:length(varargin{cnt})-1,varargin{cnt}, ...
             'Color',color{(cnt-1)/3+1},'LineWidth',2)         
  end

  str = cell(1,n);
  str{1} = name1;
  time = zeros(1,n);
  time(1) = time1;
  for i=2:n
    str{i} = varargin{3*i-3};
    time(i) = varargin{3*i-4};
  end
  
  legend(str)
  title('Residuals','FontSize',12,'FontWeight','bold')
  xlabel('Iterations','FontSize',12)
  ylabel('||b-Ax||_\infty','FontSize',12)
  axis tight
  
  subplot(1,2,2)   
  tmin = min(time);
  
  barh(time/tmin*100,'stacked','y')  
  
  offset = 100;
  htext = zeros(1,n);
  for i=1:n
    time(i) = time(i)/tmin*100; % relative time in percent
    htext(i) = text(time(i)+offset,i,[str{i}, ...
                    sprintf(' - %i%%',round(time(i)))],'FontSize',10);
           
    xlim = get(gca,'xlim');
    ext=get(htext(i),'Extent');
    if time(i) + offset + ext(3) > xlim(2)
      set(gca,'xlim',[0,time(i) + ext(3) + offset]);
    end
  end
  
  title('Rechenzeitvergleich','FontSize',12,'FontWeight','bold')
  xlabel('Rechenzeit (%)','FontSize',12)
  set(gca,'YTickLabel','')
  set(gca,'YDir','reverse')